import {Component} from '@angular/core';
import {Repo} from "./interface/repo";
import {RepoService} from "./service/repo.service";
import {FormControl, ReactiveFormsModule} from "@angular/forms";
import {MatToolbar} from "@angular/material/toolbar";
import {MatIconButton} from "@angular/material/button";
import {MatFormField, MatLabel, MatSuffix} from "@angular/material/form-field";
import {MatTooltip} from "@angular/material/tooltip";
import {MatIcon} from "@angular/material/icon";
import {MatInput} from "@angular/material/input";
import {Clipboard} from '@angular/cdk/clipboard';
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell,
  MatHeaderCellDef,
  MatHeaderRow, MatHeaderRowDef, MatRow, MatRowDef,
  MatTable
} from "@angular/material/table";
import {DatePipe} from "@angular/common";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {MatChipOption} from "@angular/material/chips";
import {MatDialog} from "@angular/material/dialog";
import {DownloadImageComponent} from "./component/download-image/download-image.component";
import {CapacityPipe} from "./pipe/capacity.pipe";

@Component({
  selector: 'app-root',
  standalone: true,
  imports: [
    MatToolbar,
    MatIconButton,
    MatSuffix,
    MatTooltip,
    MatIcon,
    MatFormField,
    MatInput,
    ReactiveFormsModule,
    MatTable,
    MatColumnDef,
    MatHeaderCell,
    MatHeaderCellDef,
    MatCell,
    MatCellDef,
    MatHeaderRow,
    MatRow,
    MatHeaderRowDef,
    MatRowDef,
    MatLabel,
    DatePipe,
    MatChipOption,
    CapacityPipe,
  ],
  templateUrl: './app.component.html',
  styleUrl: './app.component.scss',
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class AppComponent {
  title = 'front';
  columnsToDisplay: string[] = ['id', 'updated_at', "repo", "tag", 'size', 'operation'];
  columnsToDisplayWithExpand = ['expand', ...this.columnsToDisplay];
  expandedElement: Repo | null = null;
  keyword = new FormControl('');

  constructor(
    public repo_svc: RepoService,
    private clipboard: Clipboard,
    public dialog: MatDialog,
  ) {
    this.downloadImage()
  }

  search(limit: number = 20, last: number = 0) {
    this.repo_svc.GetCatalog(limit, last, this.keyword.value ?? '')
  }

  showTags(event: Event, element: Repo) {
    if (!(this.expandedElement === element)) {
      this.repo_svc.GetTags(element.repo)
    }

    this.expandedElement = this.expandedElement === element ? null : element;
    event.stopPropagation()
  }

  copyImageCommand(item: Repo) {
    this.clipboard.copy(`docker pull ${this.repo_svc.settings().base_address}/${item.repo}:${item.tag}`)
  }

  downloadImage() {
    this.dialog.open(DownloadImageComponent, {data: {}})
  }
}
