import {Component} from '@angular/core';
import {
  MatDialogActions,
  MatDialogClose,
  MatDialogContent,
  MatDialogRef,
  MatDialogTitle
} from "@angular/material/dialog";
import {MatButton} from "@angular/material/button";
import {MatFormField, MatLabel, MatPrefix, MatSuffix} from "@angular/material/form-field";
import {MatInput} from "@angular/material/input";
import {MatIcon} from "@angular/material/icon";
import {RepoService} from "../../service/repo.service";
import {HttpClient} from "@angular/common/http";
import {FormsModule} from "@angular/forms";
import {CdkConnectedOverlay, CdkOverlayOrigin} from "@angular/cdk/overlay";
import {MatProgressSpinner} from "@angular/material/progress-spinner";
import {MsgService} from "../../service/msg.service";

@Component({
  selector: 'app-download-image',
  standalone: true,
  imports: [
    MatDialogTitle,
    MatDialogContent,
    MatDialogActions,
    MatDialogClose,
    MatButton,
    MatLabel,
    MatInput,
    MatSuffix,
    MatIcon,
    MatFormField,
    MatPrefix,
    FormsModule,
    CdkConnectedOverlay,
    MatProgressSpinner,
    CdkOverlayOrigin
  ],
  templateUrl: './download-image.component.html',
  styleUrl: './download-image.component.scss'
})
export class DownloadImageComponent {

  target = ''
  source = ''
  proxy = ''
  title = '下载云上镜像'
  inProgress = false

  readonly url_proxy_download = '/api/repo/proxy'

  constructor(
    public repo_svc: RepoService,
    private dialogRef: MatDialogRef<DownloadImageComponent>,
    private msg_svc: MsgService,
  ) {
    let evt = new EventSource(this.url_proxy_download)
    evt.onmessage = (ev: MessageEvent) => {
      console.log('[D] event source on message =', ev)
    }
    evt.onerror = (ev: Event) => {
      console.log('[D] event source on error=', ev)
    }
  }

  async proxyDownload() {
    const response = await fetch(this.url_proxy_download, {
      method: "POST",
      body: JSON.stringify({target: this.target, source: this.source, proxy: this.proxy}),
      headers: {"Content-Type": "application/json"},
    })
    const reader = response.body?.pipeThrough(new TextDecoderStream()).getReader()
    if (reader) {
      this.inProgress = true
      while (true) {
        const {value, done} = await reader.read()
        if (done) {
          break;
        }

        if (value) {
          const msgs = value.split('\n')
          if (msgs.length > 2) {
            const sse_type = msgs[0].slice(6)
            const sse_data = msgs[1].slice(5)
            const sse_json = JSON.parse(sse_data)
            if (sse_json['msg']) {
              this.title = sse_json['msg']
            }

            if (sse_json['action'] && sse_json['action'] === 'done') {
              this.repo_svc.GetCatalog(this.repo_svc.catalog().limit, this.repo_svc.catalog().last, this.repo_svc.catalog().keyword)
              this.msg_svc.success('获取镜像成功')
              this.dialogRef.close()
            }
          }
        }
      }

    }

    this.inProgress = false
  }

}
