package handler

import (
	"github.com/loveuer/nf"
	"net/http"
	"nf-repo/internal/interfaces"
	"nf-repo/internal/model"
	"nf-repo/internal/util/rerr"
	"strings"
)

func handleTags(ctx *nf.Ctx, m interfaces.ManifestHandler) error {
	if ctx.Method() != "GET" {
		return rerr.Error(ctx, &rerr.RepositoryError{
			Status:  http.StatusBadRequest,
			Code:    "METHOD_UNKNOWN",
			Message: "We don't understand your method + url",
		})
	}

	type Req struct {
		Last int `json:"last" query:"last"`
		N    int `json:"n" query:"n"`
	}

	elem := strings.Split(ctx.Path(), "/")
	elem = elem[1:]
	repo := strings.Join(elem[1:len(elem)-2], "/")

	var (
		err  error
		req  = new(Req)
		re   *rerr.RepositoryError
		list *model.Tag
	)

	if err = ctx.QueryParser(req); err != nil {
		return rerr.Error(ctx, &rerr.RepositoryError{
			Status:  http.StatusBadRequest,
			Code:    "BAD_REQUEST",
			Message: err.Error(),
		})
	}

	if req.N <= 0 {
		req.N = 100
	}

	if list, re = m.Tags(ctx.Request.Context(), repo, req.N, req.Last, ""); err != nil {
		return rerr.Error(ctx, re)
	}

	return ctx.JSON(list)
}
